#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <cstdlib>
#include <utility>
#include <cstdio>

class Aggregator{
public:
    Aggregator(std::vector<float>* data, int gsize, bool debug=false)
        : m_data(data), m_gsize(gsize)
    {
        m_debug_data = NULL;
        if(debug)
            m_debug_data = new std::vector<float>(*data);
    }
    void aggregate() 
    {
        std::vector<float>* new_data = new std::vector<float>();
        for(int i=0; i+m_gsize<=m_data->size(); i++)
            new_data->push_back(group_agg(i));
        m_data = new_data;
        delete new_data;
    }
protected:
    virtual float group_agg(int index) = 0;
    std::vector<float>* m_data;
    int m_gsize;

    std::vector<float>* m_debug_data;
};

class AggregatorSum : public Aggregator{
public:
    AggregatorSum(std::vector<float>* data, int gsize)
        : Aggregator(data, gsize)
    {}
private:
    virtual float group_agg(int index)
    {
        float result = 0;
        for(int i=index; i<index+m_gsize; i++)
            result += m_data->at(i);
        return result;
    }
};

class AggregatorMean : public Aggregator{
public:
    AggregatorMean(std::vector<float>* data, int gsize)
        : Aggregator(data, gsize)
    {}
private:
    virtual float group_agg(int index)
    {
        float result = 0;
        for(int i=index; i<index+m_gsize; i++)
            result += m_data->at(i);
        return result / m_gsize;
    }
};

std::vector<float> load(const std::string& filename)
{
    FILE* input = fopen(filename.c_str(), "r");
    if(input == NULL)
    {
        std::cout << "Failed to load vector from file :(" << std::endl;
        exit(1);
    }
    std::vector<float> result;
    float value;
    while(fscanf(input, "%f", &value) == 1)
        result.push_back(value);
    return result;
}

class Sorter{
public:
    Sorter(std::vector<float>* data)
        : m_data(data)
    {}
    void sort()
    {
        *m_data = sorted(*m_data);
    }
private:
    std::vector<float> sorted(std::vector<float> data)
    {
        if(data.size() <= 1)
            return data;
        auto[data_left, data_right] = split(data);
        data_left  = sorted(data_left);
        data_right = sorted(data_right);
        return merge(data_left, data_right);
    }
    std::pair<std::vector<float>, std::vector<float> > split(std::vector<float> data)
    {
        int half_size = data.size() / 2;
        const std::vector<float>::iterator start  = data.begin();
        const std::vector<float>::iterator middle = data.begin() + half_size;
        const std::vector<float>::iterator end    = data.end();
        
        std::vector<float> left(start, middle);
        std::vector<float> right(middle, end);

        return {left, right};
    }
    std::vector<float> merge(std::vector<float> left, std::vector<float> right)
    {
        std::vector<float> result;
        std::vector<float>::iterator left_it = left.begin();
        std::vector<float>::iterator right_it = right.begin();

        while(left_it != left.end() && right_it != right.end())
        {
            if(*left_it < *right_it)
            {
                result.push_back(*left_it);
                left_it++;
            }
            else
            {
                result.push_back(*right_it);
                right_it++;
            }
        }
        while(left_it != left.end())
        {
            result.push_back(*left_it);
            left_it++;
        }
        while(right_it != right.end())
        {
            result.push_back(*right_it);
            right_it++;
        }

        return result;
    }
    std::vector<float>* m_data;
};

int main()
{
    std::vector<float> data = load(std::string("input.txt"));
    std::vector<Aggregator*> aggregators = {
        new AggregatorSum(&data, 5),
        new AggregatorMean(&data, 3)
    };
    Sorter sorter(&data);

    for(Aggregator* agg: aggregators)
        agg->aggregate();
    sorter.sort();

    for(float value: data)
        std::cout << value << " ";
    std::cout << std::endl;

    return 0;
}